/*
 * unity-webapps-indicator-view-libindicate.c
 * Copyright (C) Canonical LTD 2011
 *
 * Author: Robert Carr <racarr@canonical.com>
 * 
 unity-webapps is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * unity-webapps is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.";
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gstdio.h>
#include <gio/gio.h>

#include <libindicate/server.h>
#include <libindicate-gtk/indicator.h>
#include <libindicate/indicator.h>

#include "unity-webapps-indicator-view-libindicate.h"
#include "unity-webapps-action-manager.h"
#include "unity-webapps-resource-cache.h"
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "../unity-webapps-debug.h"

G_DEFINE_TYPE(UnityWebappsIndicatorViewLibindicate, unity_webapps_indicator_view_libindicate, UNITY_WEBAPPS_TYPE_INDICATOR_VIEW)

enum {
  PROP_0,
  PROP_DESKTOP_FILE,
  PROP_MENU_SERVER
};

struct _UnityWebappsIndicatorViewLibindicatePrivate {
  gchar *desktop_file;
  IndicateServer *server;
  
  DbusmenuServer *menu_server;
  GHashTable *indicators_by_name;
  
  GList *indicators_for_hiding;
};


#define UNITY_WEBAPPS_INDICATOR_VIEW_LIBINDICATE_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), UNITY_WEBAPPS_TYPE_INDICATOR_VIEW_LIBINDICATE, UnityWebappsIndicatorViewLibindicatePrivate))

static void unity_webapps_indicator_view_libindicate_create_server (UnityWebappsIndicatorViewLibindicate *view);
static IndicateServer *unity_webapps_indicator_view_libindicate_get_server (UnityWebappsIndicatorViewLibindicate *view);

static void
unity_webapps_indicator_view_libindicate_server_displayed (IndicateServer *server,
								    guint timestamp,
								    gpointer user_data)
{
  UnityWebappsIndicatorView *view;
  
  view = UNITY_WEBAPPS_INDICATOR_VIEW (user_data);
  
  g_signal_emit_by_name (view, "server-raised");
}

static void
unity_webapps_indicator_view_libindicate_indicator_displayed (IndicateIndicator *indicator,
								       guint timestamp,
								       gpointer user_data)
{
  UnityWebappsIndicatorViewLibindicate *view;
  GHashTableIter iter;
  gpointer key, value;
  gboolean found;
  
  view = (UnityWebappsIndicatorViewLibindicate *)user_data;
  
  g_hash_table_iter_init (&iter, view->priv->indicators_by_name);
  
  found = FALSE;
  
  while (g_hash_table_iter_next (&iter, &key, &value))
    {
      if (value == indicator)
	{
	  found = TRUE;
	  break;
	}
    }
  
  if (found)
    {
      g_signal_emit_by_name (view, "indicator-raised", (gchar *)key);
    }

  indicate_indicator_set_property_bool (indicator, "draw-attention", FALSE);
}

static void
unity_webapps_indicator_view_libindicate_create_server (UnityWebappsIndicatorViewLibindicate *view)
{
  indicate_server_ref_default (); // HACK (libindicate race bug with bus_get_cb from the default server)
  
  view->priv->server = g_object_new (INDICATE_TYPE_SERVER, NULL);
  
  g_signal_connect_object (view->priv->server,
			   "server-display",
			   G_CALLBACK (unity_webapps_indicator_view_libindicate_server_displayed),
			   view, 0);
  
  indicate_server_set_desktop_file (view->priv->server, view->priv->desktop_file);
  indicate_server_set_type (view->priv->server, "message.im");
  
  indicate_server_set_icon_theme (view->priv->server, "unity-webapps");
  
  if (view->priv->menu_server)
    {
      indicate_server_set_menu (view->priv->server, view->priv->menu_server);
    }
  
  indicate_server_show (view->priv->server);
}

static IndicateServer *
unity_webapps_indicator_view_libindicate_get_server (UnityWebappsIndicatorViewLibindicate *view)
{
  if (view->priv->server == NULL)
    {
      unity_webapps_indicator_view_libindicate_create_server (view);
    }
  
  return view->priv->server;
}

static void
unity_webapps_indicator_view_libindicate_show (UnityWebappsIndicatorView *indicator_view)
{
  UnityWebappsIndicatorViewLibindicate *self;
  IndicateServer *server;

  self = UNITY_WEBAPPS_INDICATOR_VIEW_LIBINDICATE (indicator_view);
  
  server = unity_webapps_indicator_view_libindicate_get_server (self);
  
  indicate_server_show (server);
}

static void
unity_webapps_indicator_view_libindicate_show_indicator (UnityWebappsIndicatorView *indicator_view,
							 const gchar *name)
{
  UnityWebappsIndicatorViewLibindicate *self;
  IndicateServer *server;
  IndicateIndicator *indicator;

  self = UNITY_WEBAPPS_INDICATOR_VIEW_LIBINDICATE (indicator_view);
  
  if (g_list_find_custom (self->priv->indicators_for_hiding, name, (GCompareFunc)g_strcmp0))
    {
      GList *i = self->priv->indicators_for_hiding, *next;

      while (i)
        {
          next = g_list_next (i);
          if (g_strcmp0 (name, i->data) == 0)
            {
              g_free (i->data);
              self->priv->indicators_for_hiding = g_list_delete_link (self->priv->indicators_for_hiding, i);
            }
          i = next;
        }
    }

  
  indicator = (IndicateIndicator *)g_hash_table_lookup (self->priv->indicators_by_name, name);
  
  if (indicator != NULL)
    {
      indicate_indicator_show (indicator);
      
      return;
    }
  server = unity_webapps_indicator_view_libindicate_get_server (self);
  indicator = indicate_indicator_new_with_server (server);
  
  indicate_indicator_set_property (indicator, "name", name);
  
  indicate_indicator_show (indicator);
  indicate_server_show (server);
  
  g_signal_connect_object (indicator, "user-display",
			   G_CALLBACK (unity_webapps_indicator_view_libindicate_indicator_displayed),
			   self, 0);

  g_hash_table_insert (self->priv->indicators_by_name, g_strdup (name), indicator);
}

static void
unity_webapps_indicator_view_libindicate_real_clear_indicator (UnityWebappsIndicatorViewLibindicate *self,
							       const gchar *name)
{
  g_hash_table_remove (self->priv->indicators_by_name, name);
}

static gboolean
indicators_gc (gpointer user_data)
{
  UnityWebappsIndicatorViewLibindicate *view;
  GList *i;

  view = UNITY_WEBAPPS_INDICATOR_VIEW_LIBINDICATE (user_data);
  
  for (i = view->priv->indicators_for_hiding; i; i = i->next)
  {
    unity_webapps_indicator_view_libindicate_real_clear_indicator (view, (const gchar *)i->data);
    g_free (i->data);
  }
  
  g_list_free (view->priv->indicators_for_hiding);
  view->priv->indicators_for_hiding = NULL;
  
  return FALSE;
}


static void
unity_webapps_indicator_view_libindicate_clear_indicator (UnityWebappsIndicatorView *indicator_view,
							  const gchar *name)
{
  UnityWebappsIndicatorViewLibindicate *self;
  
  self = UNITY_WEBAPPS_INDICATOR_VIEW_LIBINDICATE (indicator_view);
  
  self->priv->indicators_for_hiding = g_list_append (self->priv->indicators_for_hiding,
						     g_strdup (name));

  g_timeout_add_seconds (1, (GSourceFunc)indicators_gc, self);
}

static void
set_property_icon (IndicateIndicator *indicator,
		   const gchar *property,
		   const gchar *icon_url)
{
  GFile *icon_file;
  GFileInputStream *input_stream;
  GdkPixbuf *pb;
  gchar *icon_path;
  GError *error;
  
  icon_path = unity_webapps_resource_cache_lookup_uri (icon_url);
  
  if (icon_path == NULL)
    {
      UNITY_WEBAPPS_NOTE (INDICATOR, "Resource cache returned NULL for URL (%s)", icon_url);
      return;
    }
  
  icon_file = g_file_new_for_path (icon_path);
  
  if (icon_file == NULL)
    {
      UNITY_WEBAPPS_NOTE (INDICATOR, "Failed to construct file from icon path: %s", icon_path);
      return;
    }
  
  error = NULL;
  input_stream = g_file_read (icon_file, NULL, &error);
  
  if (error != NULL)
    {
      g_warning ("Error downloading icon url in indicator context: %s",
		 icon_url);

      g_error_free (error);
      
      g_object_unref (G_OBJECT (icon_file));
      
      return;
    }

  pb = gdk_pixbuf_new_from_stream (G_INPUT_STREAM (input_stream), NULL, &error);
  
  if (error != NULL)
    {
      g_warning ("Could not create pixbuf from image stream (%s): %s", icon_url, error->message);
      g_error_free (error);
      g_object_unref (G_OBJECT (icon_file));
      g_object_unref (G_OBJECT (input_stream));
      
      return;
    }
  
  indicate_gtk_indicator_set_property_icon (indicator, property, pb);
  
  g_object_unref (pb);
  g_object_unref (G_OBJECT (icon_file));
  g_object_unref (G_OBJECT (input_stream));
}

static void
unity_webapps_indicator_view_libindicate_set_indicator_property (UnityWebappsIndicatorView *indicator_view,
								 const gchar *indicator_name,
								 const gchar *property_name,
								 GVariant *value,
								 gboolean draw_attention)
{
  UnityWebappsIndicatorViewLibindicate *self;
  IndicateIndicator *indicator;
  
  self = UNITY_WEBAPPS_INDICATOR_VIEW_LIBINDICATE (indicator_view);
  
  indicator = (IndicateIndicator *)g_hash_table_lookup (self->priv->indicators_by_name,
							indicator_name);
  
  if (indicator == NULL)
    {
      return;
    }
  
  indicate_indicator_set_property_bool (indicator, "draw-attention", draw_attention);

  if (g_strcmp0 (property_name, "icon") == 0)
    {
      set_property_icon (indicator, property_name, g_variant_get_string(value, NULL));
    }
  else
    {
      indicate_indicator_set_property_variant (indicator, property_name, value);
    }
}

static void
unity_webapps_indicator_view_libindicate_finalize (GObject *object)
{
  UnityWebappsIndicatorViewLibindicate *view;
  
  view = (UnityWebappsIndicatorViewLibindicate *)object;

  g_hash_table_destroy (view->priv->indicators_by_name);

  G_OBJECT_CLASS (unity_webapps_indicator_view_libindicate_parent_class)->finalize(object);
}

static void
unity_webapps_indicator_view_libindicate_dispose (GObject *object)
{
  UnityWebappsIndicatorViewLibindicate *view;
  
  view = UNITY_WEBAPPS_INDICATOR_VIEW_LIBINDICATE (object);
  
  if (view->priv->menu_server)
    {
      g_object_unref (G_OBJECT (view->priv->menu_server));
      view->priv->menu_server = NULL;
    }
  
  if (view->priv->server)
    {
      g_object_unref (G_OBJECT (view->priv->server));
      view->priv->server = NULL;
    }

  G_OBJECT_CLASS (unity_webapps_indicator_view_libindicate_parent_class)->dispose(object);
}

static void
unity_webapps_indicator_view_libindicate_get_property (GObject *object,
						       guint prop_id,
						       GValue *value,
						       GParamSpec *pspec)
{
  UnityWebappsIndicatorViewLibindicate *view = UNITY_WEBAPPS_INDICATOR_VIEW_LIBINDICATE (object);
  
  switch (prop_id)
    {
    case PROP_DESKTOP_FILE:
      g_value_set_string (value, view->priv->desktop_file);
      break;
    case PROP_MENU_SERVER:
      g_value_set_object (value, view->priv->menu_server);
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void
unity_webapps_indicator_view_libindicate_set_property (GObject *object,
						       guint prop_id,
						       const GValue *value,
						       GParamSpec *pspec)
{
  UnityWebappsIndicatorViewLibindicate *view = UNITY_WEBAPPS_INDICATOR_VIEW_LIBINDICATE (object);
  
  switch (prop_id)
    {
    case PROP_DESKTOP_FILE:
      g_return_if_fail (view->priv->desktop_file == NULL);
      view->priv->desktop_file = g_value_dup_string (value);
      break;
    case PROP_MENU_SERVER:
      if (view->priv->menu_server != NULL)
	{
	  g_object_unref (G_OBJECT (view->priv->menu_server));
	}
      view->priv->menu_server = g_value_dup_object (value);
      if (view->priv->server != NULL)
	{
	  indicate_server_set_menu (view->priv->server,
				    view->priv->menu_server);
	}
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void
unity_webapps_indicator_view_libindicate_class_init (UnityWebappsIndicatorViewLibindicateClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  UnityWebappsIndicatorViewClass *indicator_view_class = UNITY_WEBAPPS_INDICATOR_VIEW_CLASS (klass);
  
  g_type_class_add_private (object_class, sizeof(UnityWebappsIndicatorViewLibindicatePrivate));
  
  object_class->finalize = unity_webapps_indicator_view_libindicate_finalize;
  object_class->dispose = unity_webapps_indicator_view_libindicate_dispose;
  
  object_class->get_property = unity_webapps_indicator_view_libindicate_get_property;
  object_class->set_property = unity_webapps_indicator_view_libindicate_set_property;
  
  indicator_view_class->show = unity_webapps_indicator_view_libindicate_show;;
  indicator_view_class->show_indicator = unity_webapps_indicator_view_libindicate_show_indicator;
  indicator_view_class->clear_indicator = unity_webapps_indicator_view_libindicate_clear_indicator;
  indicator_view_class->set_indicator_property = unity_webapps_indicator_view_libindicate_set_indicator_property;
  
  g_object_class_install_property (object_class, PROP_DESKTOP_FILE, g_param_spec_string ("desktop-file", "Desktop File",
											 "Desktop File to use for the IndicateServer",
											 NULL, 
											 G_PARAM_READWRITE |
											 G_PARAM_CONSTRUCT_ONLY |
											 G_PARAM_STATIC_STRINGS));
  g_object_class_install_property (object_class, PROP_MENU_SERVER, g_param_spec_object ("menu-server", "Menu Server",
											"Menu Server for Indicator",
											DBUSMENU_TYPE_SERVER,
											G_PARAM_READWRITE |
											G_PARAM_STATIC_STRINGS));
}

static void
unity_webapps_indicator_view_libindicate_init (UnityWebappsIndicatorViewLibindicate *view)
{
  view->priv = UNITY_WEBAPPS_INDICATOR_VIEW_LIBINDICATE_GET_PRIVATE (view);
  
  view->priv->server = NULL;
  view->priv->desktop_file = NULL;
  
  view->priv->indicators_for_hiding = NULL;
  
  view->priv->indicators_by_name = g_hash_table_new_full (g_str_hash, g_str_equal,
							  g_free, g_object_unref);
}

UnityWebappsIndicatorView *
unity_webapps_indicator_view_libindicate_new (UnityWebappsIndicatorModel *model, const gchar *desktop_file)
{
  return g_object_new (UNITY_WEBAPPS_TYPE_INDICATOR_VIEW_LIBINDICATE,
		       "model", model,
		       "desktop-file", desktop_file,
		       NULL);
}

void
unity_webapps_indicator_view_libindicate_set_menu_server (UnityWebappsIndicatorViewLibindicate *view,
							  DbusmenuServer *menu_server)
{
  g_object_set (view, "menu-server", menu_server, NULL);
}
