#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <glib-object.h>
#include <glib.h>

#include "unity-webapps-script-repo.h"

const gchar * TEST_REPO_FOLDER = "scripts";
gchar * get_repo_fullpath (void)
{
  return g_build_path (G_DIR_SEPARATOR_S, g_get_current_dir (), TEST_REPO_FOLDER, NULL);
}

const gchar * PATH_SEP = ";";

void test_get_path_no_user_defined_path (void);
void test_path_contains_non_empty_elements (void);


void test_path_contains_non_empty_elements (void)
{
  test_get_path_no_user_defined_path ();

  gchar path [1024] = {0};
  unity_webapps_script_repo_get_path (&path[0], sizeof(path));

  gchar ** paths = g_strsplit (path, PATH_SEP, 0);

  // at least one
  g_assert (NULL != paths);
  g_assert (NULL != paths[0]);

  g_strfreev (paths);
}

void test_get_path_no_user_defined_path (void)
{
  gchar path [1024] = {0};
  g_assert (0 == unity_webapps_script_repo_get_path (&path[0], sizeof(path)));
  g_assert (0 != strlen(path));
}

void test_path_contains_user_defined (void)
{
  gchar * defined_repo_path = get_repo_fullpath ();
  g_setenv ("UNITY_WEBAPPS_SCRIPT_REPO_PATH", defined_repo_path, TRUE);

  gchar path [1024] = {0};
  unity_webapps_script_repo_get_path (&path[0], sizeof(path));

  gchar ** paths = g_strsplit (path, PATH_SEP, 0);

  // at least one
  g_assert (NULL != paths);

  gboolean user_defined_repo_found = FALSE;

  gsize idx = 0;
  while (paths[idx] != NULL || ! user_defined_repo_found)
    {
      user_defined_repo_found = (0 == g_strcmp0 (paths[idx], defined_repo_path));
    
      ++idx;
    }

  g_strfreev (paths);
  g_unsetenv ("UNITY_WEBAPPS_SCRIPT_REPO_PATH");
  g_free (defined_repo_path);

  g_assert (user_defined_repo_found);
}

gint
main (gint argc, gchar **argv)
{
  g_type_init();

  g_message ("--- Running: test_get_path_no_user_defined_path");
  test_get_path_no_user_defined_path();

  g_message ("--- Running: test_path_contains_non_empty_elements");
  test_path_contains_non_empty_elements ();

  g_message ("--- Running: test_path_contains_user_defined");
  test_path_contains_user_defined ();

  return 0;
}
