#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <glib.h>

#include "unity-webapps-script-repo.h"

const gchar * TEST_REPO_FOLDER = "scripts";
gchar * get_repo_fullpath (void)
{
  return g_build_path (G_DIR_SEPARATOR_S, g_get_current_dir (), TEST_REPO_FOLDER, NULL);
}

#define TEST_EXPECTED_OVERALL_SCRIPT_COUNT 5
#define TEST_EXPECTED_VALID_SCRIPT_COUNT 1

gboolean test_expected_script_count (UnityWebappsIntegrationScriptDescription ** ppDescription)
{
  g_assert (NULL != ppDescription);

  guint count = 0;
  while (ppDescription[count] != NULL) ++count;

  g_assert (TEST_EXPECTED_VALID_SCRIPT_COUNT == count);

  return TRUE;
}

void test_single_description (UnityWebappsIntegrationScriptDescription * pDescription)
{
  g_assert (NULL != pDescription->name && 0 != strlen(pDescription->name));
  g_message ("name: %s", pDescription->name);

  g_assert (NULL != pDescription->version && 0 != strlen(pDescription->version));
  g_message ("version: %s", pDescription->version);

  g_assert (NULL != pDescription->author && 0 != strlen(pDescription->author));
  g_message ("author: %s", pDescription->author);

  void print_includes (gpointer data, gpointer user_data)
  {
    g_assert (NULL != data);
    g_message ("include name: %s", (gchar*) data);
  }
  g_assert (NULL != pDescription->includes);
  g_slist_foreach (pDescription->includes, print_includes, NULL);

  if (NULL != pDescription->requires)
    {
      void print_requires (gpointer data, gpointer user_data)
      {
	UnityWebappsIntegrationScriptContent * pContent = (UnityWebappsIntegrationScriptContent *) data;

	g_assert (NULL != pContent);
	g_assert (NULL != pContent->name && NULL != pContent->content);

	g_message ("require name: %s", pContent->name);
      }
      g_slist_foreach (pDescription->requires, print_requires, NULL);
    }

  g_assert (NULL != pDescription->content);
}

gboolean test_descriptions (UnityWebappsIntegrationScriptDescription ** ppDescription)
{
  g_assert (NULL != ppDescription);

  guint idx = 0;

  while (ppDescription[idx] != NULL)
    {
      test_single_description (ppDescription[idx]);
      ++idx;
    }

  return TRUE;
}

gint
main (gint argc, gchar **argv)
{
#if defined (WITH_MEMORY_PROFILE)
  g_mem_set_vtable (glib_mem_profiler_table);
#endif

  gchar * defined_repo_path = get_repo_fullpath ();
  
  g_setenv ("UNITY_WEBAPPS_SCRIPT_REPO_PATH", defined_repo_path, TRUE);

  g_assert (TEST_EXPECTED_OVERALL_SCRIPT_COUNT == unity_webapps_script_repo_get_script_count ());

  UnityWebappsIntegrationScriptDescription
    ** ppDescription = unity_webapps_script_repo_get_descriptions ();
  g_assert (NULL != ppDescription);

  test_expected_script_count (ppDescription);

  test_descriptions (ppDescription);

  unity_webapps_script_repo_free_all (ppDescription);

  g_unsetenv ("UNITY_WEBAPPS_SCRIPT_REPO_PATH");
  g_free (defined_repo_path);

#if defined (WITH_MEMORY_PROFILE)
  g_mem_profile();
#endif

  return 0;
}
